viewHandlers.FlowAlbumView = inheritHandler('FlowAlbumView', 'BaseList', {
    controlClass: 'FlowAlbumView',
    title: _('3D Album View'),
    icon: 'browser',
    dataSourceType: 'albumlist',
    onShow: function (control, view) {
        
        // console.log('onShow called');
        var handler = getNodeHandler(view);
        
        view.mainControl = control.controlClass;
        
        // special case for if this is a playlist
        if (view.viewNode.dataSource && view.viewNode && view.viewNode.handlerID === 'playlist') {
            control.controlClass.playlist = view.viewNode.dataSource;
        }
        else control.controlClass.playlist = null;
        
        if (!this.dataSourceType)
            alert('dataSourceType not defined in viewHandlers.' + this.constructor.name + ' (descendant of ImageGrid)');
        if (view.dataSourceCache[this.dataSourceType]) {
            control.controlClass.dataSource = view.dataSourceCache[this.dataSourceType];
            
            // #18722: for switching view types in an already loaded playlist
            if (control.controlClass.playlist) {
                if (view.dataSourceCache['tracklist']) {
                    // console.log('getting from cache');
                    ODS('FlowAlbumView - Getting tracklist from cache (is a playlist)');
                    control.controlClass.completeTracklist = view.dataSourceCache['tracklist'];
                }
                else {
                    // console.log('getting from playlist');
                    ODS('FlowAlbumView - Getting tracklist from playlist');
                    let tracklist = control.controlClass.playlist.getTracklist();
                    tracklist.whenLoaded()
                    .then(() => {
                        control.controlClass.completeTracklist = tracklist;
                    })
                }
            }
        }
        else
            if (handler.getViewDataSource) {
                var ds = handler.getViewDataSource(view, this.dataSourceType);
                view.dataSourceCache[this.dataSourceType] = ds;
                view.delayedAssign(ds, function () {
                    control.controlClass.dataSource = ds;
                });
            }
            else if (!view.viewNode.dataSource) {
                // this is for now playing view
                var songlist = app.player.getSongList();
                var tracklist = songlist.getTracklist();
                var ds = tracklist.getAlbumList();
                view.delayedAssign(ds, function () {
                    control.controlClass.dataSource = ds;
                    control.controlClass.completeTracklist = tracklist;
                });
                view.dataSourceCache['tracklist'] = tracklist; // #18722
            }
            else if (view.viewNode.dataSource.getAlbumList)
                control.controlClass.dataSource = view.viewNode.dataSource.getAlbumList()
            else if (view.viewNode.dataSource.getTracklist) {
                var tracklist = view.viewNode.dataSource.getTracklist();
                tracklist.whenLoaded()
                .then(() => {
                    if (tracklist.getAlbumList) {
                        var ds = tracklist.getAlbumList();
                        view.dataSourceCache[this.dataSourceType] = ds;
                        view.delayedAssign(ds, function () {
                            control.controlClass.dataSource = ds;
                            control.controlClass.completeTracklist = tracklist;
                        });
                        view.dataSourceCache['tracklist'] = tracklist; // #18722
                    }
                    else console.error('tracklist.getAlbumList undefined');
                })
            }
            else {
                console.warn('Couldn\'t find data source');
            }
    },
    onHide: function (control, view) {
        // control.cleanUp();
    },
    subViews: ['FlowAlbumView_tracks', 'statusBar'],
    hiddenSubViews: ['FlowAlbumView_tracks'],
});

viewHandlers.FlowAlbumView_tracks = inheritHandler('FlowAlbumView_tracks', 'TracklistBase', {
    controlClass: 'ColumnTrackList',
    title: function () {
        return _('Tracks');
    },
    icon: 'song',
    placement: {
        position: 'bottom',
        hasSplitter: true,
        inScroller: false,
    },
    onShow: function (control, view) {
        // make it accessible to the FlowAlbumView control via parentNode
        view.tracklistControl = control.controlClass;
        
        // control.classList.add('threeDView-tracklist');
        
        // The following is copied from the ColumnTracklist control
        if (this.controlClass_initParams && this.controlClass_initParams.showHeader == false)
            control.controlClass.showHeader = false;
        else
            control.controlClass.showHeader = true;

        var sortString = window.uitools.getDefaultColumnSort(view);
        if (sortString) {
            if (sortString == 'none')
                sortString = '';
            control.controlClass.getDefaultSortString = function () {
                return sortString;
            };
            control.controlClass.setSortColumns(sortString);
        }

        viewHandlersClasses['Tracklist'].$super.onShow.apply(this, arguments);
        
        if (view.mainControl && view.mainControl.tracklistDataSource) {
            view.mainControl.refreshTracklistDS();
            control.controlClass.dataSource = view.mainControl.tracklistDataSource;
        }
        else {
            // Initialize the list as empty so it does not default to all tracks
            control.controlClass.dataSource = utils.createSharedList();
        }
        
        if (view.mainControl) {
            // Force the view to render a few extra frames, so that you don't get the ugly stretching effect
            view.mainControl.triggerSmoothResize(animTools.animationTime * 1000);
            // Force the 3DController to grab the tracklist
            if (view.mainControl.controller) view.mainControl.controller._updateCurrentAlbum(true);
        } 
    },
    onHide: function (control, view) {
        view.tracklistControl = undefined;
        viewHandlersClasses['Tracklist'].$super.onHide.apply(this, arguments);
        
        if (view.mainControl) {
            view.mainControl.clearTracklistDS();
            // Force the view to render a few extra frames, so that you don't get the ugly stretching effect
            view.mainControl.triggerSmoothResize(animTools.animationTime * 1000);
        } 
    },
    // These params are necessary for the tracklist view to open smoothly, at a specified height
    controlClass_initParams: {
        minHeight: function () { // use function, so it is computed later, when document.body always exists
            return 3 * fontLineSizePx() + 'px';
        },
        height: function () {
            return 14 * fontLineSizePx() + 'px';
        },
    },    
});



viewHandlers.FlowAlbumViewNowPlaying = inheritHandler('FlowAlbumViewNowPlaying', 'FlowAlbumView', {
    controlClass: 'FlowAlbumViewNowPlaying',
    title: _('3D Album View'),
    icon: 'browser',
    dataSourceType: 'albumlist',
    onShow: function (control, view) {
        view.mainControl = control.controlClass;
        
        this.refreshDataSource = (async function () {
            var songlist = await app.player.getSongList().whenLoaded();
            if (songlist.whenMetadataLoaded)
                await songlist.whenMetadataLoaded();
            var tracklist = await songlist.getTracklist().whenLoaded();
            var ds = await tracklist.getAlbumList().whenLoaded();
            
            // console.log(ds.count);
            // console.log(songlist.count);
            // console.log(tracklist.count);
            control.controlClass.dataSource = ds;
            control.controlClass.completeTracklist = tracklist;
            // Set it to the right focused album
            if (control.controlClass.handlePlaybackStateChange) {
                control.controlClass.handlePlaybackStateChange('play');
            }
        }).bind(this);
        
        // Refresh data source both now and when the now playing list has been modified
        app.listen(app.player, 'nowPlayingModified', this.refreshDataSource);
        this.refreshDataSource();
        setTimeout(this.refreshDataSource, 8000);
    },
    onHide: function (control, view) {
        app.unlisten(app.player, 'nowPlayingModified', this.refreshDataSource);
    }
});

nodeHandlers.albums.viewAs.push('FlowAlbumView');
nodeHandlers.playlist.viewAs.push('FlowAlbumView');
nodeHandlers.tracks.viewAs.push('FlowAlbumView');